@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package Getopt::Long::Descriptive;
 # ABSTRACT: Getopt::Long, but simpler and more powerful
-$Getopt::Long::Descriptive::VERSION = '0.098';
+$Getopt::Long::Descriptive::VERSION = '0.101';
 use Carp qw(carp croak);
 use File::Basename ();
 use Getopt::Long 2.33;
@@ -23,7 +23,7 @@ use Getopt::Long::Descriptive::Usage;
 #pod     [ 'port|p=i',   "the port to connect to",   { default  => 79 } ],
 #pod     [],
 #pod     [ 'verbose|v',  "print extra stuff"            ],
-#pod     [ 'help',       "print usage message and exit" ],
+#pod     [ 'help',       "print usage message and exit", { shortcircuit => 1 } ],
 #pod   );
 #pod
 #pod   print($usage->text), exit if $opt->help;
@@ -304,12 +304,20 @@ my %HIDDEN = (
   hidden => 1,
 );
 
-my $SPEC_RE = qr{(?:[:=][\d\w\+]+[%@]?({\d*,\d*})?|[!+])$};
+my $SPEC_RE = qr{(?:[:=][0-9\w\+]+[%@]?(\{[0-9]*,[0-9]*\})?|[!+])$};
 sub _strip_assignment {
   my ($self, $str) = @_;
 
   (my $copy = $str) =~ s{$SPEC_RE}{};
 
+  if (wantarray) {
+      my $len = length $copy;
+      my $assignment = substr $str, $len;
+      if (!defined($assignment)) {
+          $assignment = '';
+      }
+      return ($copy, $assignment);
+  }
   return $copy;
 }
 
@@ -330,6 +338,8 @@ sub _build_describe_options {
     my $arg    = (ref $_[-1] and ref $_[-1] eq 'HASH') ? pop @_ : {};
     my @opts;
 
+    my %parent_of;
+
     # special casing
     # wish we had real loop objects
     my %method_map;
@@ -347,6 +357,7 @@ sub _build_describe_options {
         for my $one_opt (_expand(
           @{delete $opt->{constraint}->{one_of}}
         )) {
+          $parent_of{$one_opt->{name}} = $opt->{name};
           $one_opt->{constraint}->{implies}
             ->{$opt->{name}} = $one_opt->{name};
           for my $wipe (qw(required default)) {
@@ -396,7 +407,7 @@ sub _build_describe_options {
       sort  { lc $a cmp lc $b or $a cmp $b }
       grep  { /^.$/ }
       map   { split /\|/ }
-      map   { __PACKAGE__->_strip_assignment($_) }
+      map   { scalar __PACKAGE__->_strip_assignment($_) }
       @specs;
 
     my $long = grep /\b[^|]{2,}/, @specs;
@@ -451,6 +462,8 @@ sub _build_describe_options {
         spec   => $copt->{constraint},
         opts   => \@opts,
         usage  => $usage,
+        given_keys => \@given_keys,
+        parent_of  => \%parent_of,
       );
       next unless (defined($new) || exists($return{$name}));
       $return{$name} = $new;
@@ -485,7 +498,10 @@ sub _validate_with {
     spec   => 1,
     opts   => 1,
     usage  => 1,
+    given_keys => 1,
+    parent_of  => 1,
   });
+
   my $spec = $arg{spec};
   my %pvspec;
   for my $ct (keys %{$spec}) {
@@ -524,7 +540,11 @@ sub _validate_with {
   my %p;
   my $ok = eval {
     %p = validate_with(
-      params => [ %{$arg{params}} ],
+      params => [
+        %{$arg{params}},
+        '-given_keys', $arg{given_keys},
+        '-parent_of',  $arg{parent_of},
+      ],
       spec   => { $arg{name} => \%pvspec },
       allow_extra => 1,
       on_fail     => sub {
@@ -578,13 +598,29 @@ sub _mk_implies {
   my $whatstr = join(q{, }, map { "$_=$what->{$_}" } keys %$what);
 
   return "$name implies $whatstr" => sub {
-    my ($pv_val) = shift;
+    my ($pv_val, $rest) = @_;
 
     # negatable options will be 0 here, which is ok.
     return 1 unless defined $pv_val;
 
     while (my ($key, $val) = each %$what) {
-      if (exists $param->{$key} and $param->{$key} ne $val) {
+      # Really, this should be called "-implies" and should include all implies
+      # relationships, but they'll have to get handled by setting conflicts.
+      my $parent   = $rest->{'-parent_of'}{$name};
+      my @siblings = $parent
+                   ? (grep {; defined $rest->{'-parent_of'}{$_}
+                              && $rest->{'-parent_of'}{$_} eq $parent }
+                      @{ $rest->{'-given_keys'} })
+                   : ();
+
+      if (@siblings > 1) {
+        die "these options conflict; each wants to set the $parent: @siblings\n";
+      }
+
+      if (  exists $param->{$key}
+        and $param->{$key} ne $val
+        and grep {; $_ eq $key } @{ $rest->{'-given_keys'} }
+      ) {
         die(
           "option specification for $name implies that $key should be "
           . "set to '$val', but it is '$param->{$key}' already\n"
@@ -651,7 +687,7 @@ Getopt::Long::Descriptive - Getopt::Long, but simpler and more powerful
 
 =head1 VERSION
 
-version 0.098
+version 0.101
 
 =head1 SYNOPSIS
 
@@ -663,7 +699,7 @@ version 0.098
     [ 'port|p=i',   "the port to connect to",   { default  => 79 } ],
     [],
     [ 'verbose|v',  "print extra stuff"            ],
-    [ 'help',       "print usage message and exit" ],
+    [ 'help',       "print usage message and exit", { shortcircuit => 1 } ],
   );
 
   print($usage->text), exit if $opt->help;
@@ -937,6 +973,70 @@ Ricardo Signes <rjbs@cpan.org>
 
 =back
 
+=head1 CONTRIBUTORS
+
+=for stopwords Arthur Axel 'fREW' Schmidt Dave Rolsky Diab Jerius Hans Dieter Pearcey Harley Pig hdp@cpan.org Karen Etheridge Niels Thykier Olaf Alders Roman Hubacek Smylers Thomas Neumann zhouzhen1
+
+=over 4
+
+=item *
+
+Arthur Axel 'fREW' Schmidt <frioux@gmail.com>
+
+=item *
+
+Dave Rolsky <autarch@urth.org>
+
+=item *
+
+Diab Jerius <djerius@cfa.harvard.edu>
+
+=item *
+
+Hans Dieter Pearcey <hdp@pobox.com>
+
+=item *
+
+Hans Dieter Pearcey <hdp@weftsoar.net>
+
+=item *
+
+Harley Pig <harleypig@gmail.com>
+
+=item *
+
+hdp@cpan.org <hdp@cpan.org@fc0e91e4-031c-0410-8307-be39b06d7656>
+
+=item *
+
+Karen Etheridge <ether@cpan.org>
+
+=item *
+
+Niels Thykier <niels@thykier.net>
+
+=item *
+
+Olaf Alders <olaf@wundersolutions.com>
+
+=item *
+
+Roman Hubacek <roman.hubacek@centrum.cz>
+
+=item *
+
+Smylers <SMYLERS@cpan.fsck.com>
+
+=item *
+
+Thomas Neumann <blacky+perl@fluffbunny.de>
+
+=item *
+
+zhouzhen1 <zhouzhen1@gmail.com>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2005 by Hans Dieter Pearcey.
